# Generated by Django 5.1.1 on 2024-09-24 10:32

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("oauth2_capture", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="oauthtoken",
            name="access_token",
            field=models.TextField(help_text="The OAuth access token for API access."),
        ),
    ]



from django.urls import path

from .views import index, linkedin_post, x_post

app_name = "demo"

urlpatterns = [
    path("", index, name="index"),
    path("x/<slug:slug>/", x_post, name="x"),
    path("linkedin/<slug:slug>/", linkedin_post, name="linkedin"),
]



import logging
import secrets

from django.contrib.auth.mixins import LoginRequiredMixin
from django.db.models import QuerySet
from django.http import HttpRequest, HttpResponse
from django.shortcuts import redirect
from django.urls import reverse_lazy
from django.views.generic import DeleteView, ListView

from oauth2_capture.models import OAuthToken
from oauth2_capture.services.oauth2 import OAuth2ProviderFactory

logger = logging.getLogger(__name__)


def oauth2_callback(request: HttpRequest, provider: str) -> HttpResponse:
    """Finalize the Oauth2 flow, which is called by the OAuth2 provider after the user has authorized the app.

    Args:
        request (HttpRequest): The request object.
        provider (str): The OAuth2 provider name.

    Returns:
        HttpResponse: The response object

    """
    try:
        oauth2_provider = OAuth2ProviderFactory.get_provider(provider)
    except ValueError as e:
        return HttpResponse(str(e), status=400)

    code = request.GET.get("code")
    redirect_uri = request.build_absolute_uri(f"/oauth2_capture/{provider}/callback/")

    token_data = oauth2_provider.exchange_code_for_token(code, redirect_uri, request)

    msg = f"Token data: {token_data}"
    logger.debug(msg)

    access_token = token_data.get("access_token")

    if access_token:
        user_info = oauth2_provider.get_user_info(access_token)

        oauth_token, created = OAuthToken.objects.get_or_create(
            provider=provider,
            user_id=user_info.get("id"),
            owner=request.user,
        )
        oauth2_provider.update_token(oauth_token, token_data, user_info)

        return HttpResponse(f"{provider.capitalize()} account connected successfully.")

    return HttpResponse(
        f"Failed to connect {provider.capitalize()} account.", status=400
    )


def initiate_oauth2(request: HttpRequest, provider: str) -> HttpResponse:
    """Initiate the OAuth2 flow, which redirects the user to the OAuth2 provider's authorization page.

    Args:
        request (HttpRequest): The request object.
        provider (str): The OAuth2 provider name.

    Returns:
        HttpResponse: The response object

    """
    try:
        oauth2_provider = OAuth2ProviderFactory.get_provider(provider)
    except ValueError as e:
        return HttpResponse(str(e), status=400)

    state = secrets.token_urlsafe(32)
    redirect_uri = request.build_absolute_uri(f"/oauth2_capture/{provider}/callback/")
    auth_url = oauth2_provider.get_authorization_url(state, redirect_uri, request)

    # Store state in session for later verification
    request.session[f"{provider}_oauth_state"] = state

    logger.debug("Redirecting to %s", auth_url)
    return redirect(auth_url)


class DeleteOAuthTokenView(LoginRequiredMixin, DeleteView):
    """View for deleting an OAuth token."""

    model = OAuthToken
    success_url = reverse_lazy("oauth2_capture:list")

    def get_queryset(self) -> QuerySet:
        """Return the queryset of OAuth tokens owned by the current user."""
        return super().get_queryset().filter(owner=self.request.user)


class OAuthListView(LoginRequiredMixin, ListView):
    """View to list all the OAuth2 connections."""

    model = OAuthToken
    context_object_name = "connections"

    def get_queryset(self) -> QuerySet:
        """Get the queryset of OAuth2 connections."""
        return self.request.user.oauth_tokens.all()



from django.apps import AppConfig


class OAuth2CaptureConfig(AppConfig):
    """App configuration for the oauth2 capture app."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "oauth2_capture"



from __future__ import annotations

import base64
import hashlib
import logging
import secrets
from abc import ABC, abstractmethod
from datetime import timedelta
from typing import TYPE_CHECKING
from urllib.parse import urlencode

import requests
from django.conf import settings
from django.utils import timezone
from requests.auth import HTTPBasicAuth

if TYPE_CHECKING:
    from django.http import HttpRequest

    from oauth2_capture.models import OAuthToken


logger = logging.getLogger(__name__)


def generate_code_verifier(length: int = 128) -> str:
    """Generate a code verifier for PKCE.

    Args:
        length (int): The length of the code verifier.

    Returns:
        str: The code verifier.

    """
    return secrets.token_urlsafe(length)


def generate_code_challenge(verifier: str) -> str:
    """Generate a code challenge for PKCE.

    Args:
        verifier (str): The code verifier.

    Returns:
        str: The code challenge.

    """
    sha256 = hashlib.sha256(verifier.encode("utf-8")).digest()
    return base64.urlsafe_b64encode(sha256).decode("utf-8").rstrip("=")


class OAuth2Provider(ABC):
    """Base class for OAuth2 providers."""

    def __init__(self, provider_name: str) -> None:
        """Initialize the OAuth2 provider.

        Args:
            provider_name (str): The name of the provider.

        """
        self.provider_name = provider_name
        self.config = settings.OAUTH2_CONFIG[provider_name]

    @property
    @abstractmethod
    def authorize_url(self) -> str:
        """The URL to authorize the user."""

    @property
    @abstractmethod
    def token_url(self) -> str:
        """The URL to exchange the code for a token."""

    @property
    @abstractmethod
    def user_info_url(self) -> str:
        """The URL to get the user info."""

    @abstractmethod
    def get_user_info(self, access_token: str) -> dict:
        """Get the user info from the provider.

        Args:
            access_token (str): The access token.

        Returns:
            dict: The user info.

        """

    def get_authorization_url(
        self,
        state: str,
        redirect_uri: str,
        request: HttpRequest,  # noqa: ARG002
    ) -> str:
        """Get the authorization URL for the provider.

        Args:
            state (str): The state parameter.
            redirect_uri (str): The redirect URI.
            request (HttpRequest): The request object.

        Returns:
            str: The authorization URL.

        """
        params = {
            "client_id": self.config["client_id"],
            "redirect_uri": redirect_uri,
            "response_type": "code",
            "state": state,
            "scope": self.config["scope"],
        }
        return f"{self.authorize_url}?{urlencode(params)}"

    @abstractmethod
    def exchange_code_for_token(
        self, code: str, redirect_uri: str, request: HttpRequest
    ) -> dict:
        """Exchange the auth code for an access token.

        Args:
            code (str): The code.
            redirect_uri (str): The redirect URI.
            request (HttpRequest): The request object.

        Returns:
            dict: The token data.

        """

    def refresh_token(self, refresh_token: str) -> dict:
        """Refresh the access token.

        Args:
            refresh_token (str): The refresh token.

        Returns:
            dict: The token data.

        """
        data = {
            "grant_type": "refresh_token",
            "refresh_token": refresh_token,
            "client_id": self.config["client_id"],
            "client_secret": self.config["client_secret"],
        }

        # Encode the client_id and client_secret
        credentials = f"{self.config["client_id"]}:{self.config["client_secret"]}"
        encoded_credentials = base64.b64encode(credentials.encode()).decode()

        headers = {
            "Authorization": f"Basic {encoded_credentials}",
            "Content-Type": "application/x-www-form-urlencoded",
        }
        response = requests.post(self.token_url, data=data, headers=headers, timeout=10)
        return response.json()

    def get_valid_token(self, oauth_token: OAuthToken) -> str | None:
        """Get a valid access token for the user. If the token is expired, it will be refreshed.

        Args:
            oauth_token (OAuthToken): The user.

        Returns:
            str|None: The access token or None if there is no token.

        """
        if oauth_token.expires_at <= timezone.now():
            msg = f"Token for {oauth_token.provider}:{oauth_token.name} has expired. Refreshing..."
            logger.debug(msg)
            token_data = self.refresh_token(oauth_token.refresh_token)
            self.update_token(
                oauth_token=oauth_token,
                token_data=token_data,
                user_info=self.get_user_info(token_data["access_token"]),
            )

        logger.debug("Token for %s:%s is valid", oauth_token.provider, oauth_token.name)
        return oauth_token.access_token

    def update_token(
        self, oauth_token: OAuthToken, token_data: dict, user_info: dict
    ) -> None:
        """Update the OAuth token with the new data.

        Args:
            oauth_token (OAuthToken): The OAuth token to update.
            token_data (dict): The new token data.
            user_info (dict): The user info.

        Returns:
            None

        """
        logger.debug(token_data)

        oauth_token.access_token = token_data["access_token"]
        oauth_token.refresh_token = token_data.get(
            "refresh_token", oauth_token.refresh_token
        )

        if "expires_in" in token_data:
            oauth_token.expires_at = timezone.now() + timedelta(
                seconds=token_data["expires_in"]
            )
        if "refresh_token_expires_in" in token_data:
            oauth_token.refresh_token_expires_at = timezone.now() + timedelta(
                seconds=token_data["refresh_token_expires_in"]
            )
        oauth_token.scope = token_data.get("scope")
        oauth_token.token_type = token_data.get("token_type")

        # Update the user info
        oauth_token.profile_json = user_info
        oauth_token.name = user_info.get("name")

        oauth_token.save()


class TwitterOAuth2Provider(OAuth2Provider):
    """Twitter OAuth2 provider."""

    @property
    def authorize_url(self) -> str:
        """The URL to authorize the user."""
        return "https://twitter.com/i/oauth2/authorize"

    @property
    def token_url(self) -> str:
        """The URL to exchange the code for a token."""
        return "https://api.twitter.com/2/oauth2/token"

    @property
    def user_info_url(self) -> str:
        """The URL to get the user info."""
        return "https://api.twitter.com/2/users/me"

    def get_authorization_url(
        self, state: str, redirect_uri: str, request: HttpRequest
    ) -> str:
        """Get the authorization URL for Twitter. Override to include PKCE.

        Args:
            state (str): The state parameter.
            redirect_uri (str): The redirect URI.
            request (HttpRequest): The request object.

        Returns:
            str: The authorization URL.

        """
        # Generate code verifier and code challenge
        code_verifier = generate_code_verifier()
        code_challenge = generate_code_challenge(code_verifier)

        request.session["code_verifier"] = code_verifier

        params = {
            "client_id": self.config["client_id"],
            "redirect_uri": redirect_uri,
            "response_type": "code",
            "state": state,
            "scope": self.config["scope"],
            "code_challenge": code_challenge,
            "code_challenge_method": "S256",
        }
        return f"{self.authorize_url}?{urlencode(params)}"

    def get_user_info(self, access_token: str) -> dict:
        """Get the user info from Twitter.

        Args:
            access_token (str): The access token.

        Returns:
            dict: The user info.

        """
        headers = {"Authorization": f"Bearer {access_token}"}
        params = {"user.fields": "id,name,username,profile_image_url,description"}
        response = requests.get(
            self.user_info_url, headers=headers, params=params, timeout=10
        )
        return response.json().get("data", {})

    def exchange_code_for_token(
        self, code: str, redirect_uri: str, request: HttpRequest
    ) -> dict:
        """Exchange the auth code for an access token.

        Args:
            code (str): The code.
            redirect_uri (str): The redirect URI.
            request (HttpRequest): The request object.

        Returns:
            dict: The token data.

        """
        code_verifier = request.session.get("code_verifier")

        # Override to include PKCE
        data = {
            "grant_type": "authorization_code",
            "code": code,
            "redirect_uri": redirect_uri,
            "client_id": self.config["client_id"],
            "client_secret": self.config["client_secret"],
            "code_verifier": code_verifier,
        }

        # Use Basic Auth for Twitter
        credentials = f"{self.config['client_id']}:{self.config['client_secret']}"
        encoded_credentials = base64.b64encode(credentials.encode()).decode()
        headers = {
            "Content-Type": "application/x-www-form-urlencoded",
            "Authorization": f"Basic {encoded_credentials}",
        }

        response = requests.post(self.token_url, data=data, headers=headers, timeout=10)

        logger.debug("Twitter token response: %s", response.json())
        return response.json()


class LinkedInOAuth2Provider(OAuth2Provider):
    """LinkedIn OAuth2 provider."""

    @property
    def authorize_url(self) -> str:
        """The URL to authorize the user."""
        return "https://www.linkedin.com/oauth/v2/authorization"

    @property
    def token_url(self) -> str:
        """The URL to exchange the code for a token."""
        return "https://www.linkedin.com/oauth/v2/accessToken"

    @property
    def user_info_url(self) -> str:
        """The URL to get the user info."""
        return "https://api.linkedin.com/v2/userinfo"

    def get_user_info(self, access_token: str) -> dict:
        """Get the user info from LinkedIn.

        Args:
            access_token (str): The access token.

        Returns:
            dict: The user info.

        """
        headers = {"Authorization": f"Bearer {access_token}"}
        response = requests.get(self.user_info_url, headers=headers, timeout=10)
        res = response.json()

        # map the sub claim to id
        res["id"] = res.get("sub")

        logger.debug("LinkedIn user info: %s", res)
        return res

    def exchange_code_for_token(
        self,
        code: str,
        redirect_uri: str,
        request: HttpRequest,  # noqa: ARG002
    ) -> dict:
        """Exchange the auth code for an access token.

        Args:
            code (str): The code.
            redirect_uri (str): The redirect URI.
            request (HttpRequest): The request object.

        Returns:
            dict: The token data.

        """
        data = {
            "grant_type": "authorization_code",
            "code": code,
            "redirect_uri": redirect_uri,
            "client_id": self.config["client_id"],
            "client_secret": self.config["client_secret"],
        }
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        response = requests.post(self.token_url, data=data, headers=headers, timeout=10)
        return response.json()


class GitHubOAuth2Provider(OAuth2Provider):
    """GitHub OAuth2 provider."""

    @property
    def authorize_url(self) -> str:
        """The URL to authorize the user."""
        return "https://github.com/login/oauth/authorize"

    @property
    def token_url(self) -> str:
        """The URL to exchange the code for a token."""
        return "https://github.com/login/oauth/access_token"

    @property
    def user_info_url(self) -> str:
        """The URL to get the user info."""
        return "https://api.github.com/user"

    def get_user_info(self, access_token: str) -> dict:
        """Get the user info from GitHub.

        Args:
            access_token (str): The access token.

        Returns:
            dict: The user info.

        """
        headers = {"Authorization": f"token {access_token}"}
        response = requests.get(self.user_info_url, headers=headers, timeout=10)

        msg = f"\n\n{self.provider_name} user info response: {response.text}\n\n"
        logger.debug(msg)

        res = response.json()

        res["name"] = res["login"] if res["name"] is None else res["name"]

        return res

    def exchange_code_for_token(
        self,
        code: str,
        redirect_uri: str,
        request: HttpRequest,  # noqa: ARG002
    ) -> dict:
        """Exchange the auth code for an access token.

        Args:
            code (str): The code.
            redirect_uri (str): The redirect URI.
            request (HttpRequest): The request object.

        Returns:
            dict: The token data.

        """
        data = {
            "client_id": self.config["client_id"],
            "client_secret": self.config["client_secret"],
            "code": code,
            "redirect_uri": redirect_uri,
        }
        headers = {"Accept": "application/json"}
        response = requests.post(self.token_url, data=data, headers=headers, timeout=10)

        return response.json()


class RedditOAuth2Provider(OAuth2Provider):
    """Reddit OAuth2 provider."""

    @property
    def authorize_url(self) -> str:
        """The URL to authorize the user."""
        return "https://www.reddit.com/api/v1/authorize"

    @property
    def token_url(self) -> str:
        """The URL to exchange the code for a token."""
        return "https://www.reddit.com/api/v1/access_token"

    @property
    def user_info_url(self) -> str:
        """The URL to get the user info."""
        return "https://oauth.reddit.com/api/v1/me"

    def get_user_info(self, access_token: str) -> dict:
        """Get the user info from Reddit.

        Args:
            access_token (str): The access token.

        Returns:
            dict: The user info.

        """
        headers = {"Authorization": f"Bearer {access_token}"}
        response = requests.get(self.user_info_url, headers=headers, timeout=10)
        return response.json()

    def exchange_code_for_token(
        self,
        code: str,
        redirect_uri: str,
        request: HttpRequest,  # noqa: ARG002
    ) -> dict:
        """Exchange the auth code for an access token.

        Args:
            code (str): The code.
            redirect_uri (str): The redirect URI.
            request (HttpRequest): The request object.

        Returns:
            dict: The token data.

        """
        data = {
            "grant_type": "authorization_code",
            "code": code,
            "redirect_uri": redirect_uri,
            "client_id": self.config["client_id"],
            "client_secret": self.config["client_secret"],
        }
        headers = {
            "User-Agent": "AgentsAsylum/1.0 by simplecto",
        }

        auth = HTTPBasicAuth(self.config["client_id"], self.config["client_secret"])
        response = requests.post(
            self.token_url, data=data, auth=auth, headers=headers, timeout=10
        )

        return response.json()

    def get_authorization_url(
        self,
        state: str,
        redirect_uri: str,
        request: HttpRequest,  # noqa: ARG002
    ) -> str:
        """Get the authorization URL for Reddit.

        Args:
            state (str): The state parameter.
            redirect_uri (str): The redirect URI.
            request (HttpRequest): The request object.

        Returns:
            str: The authorization URL.

        """
        params = {
            "client_id": self.config["client_id"],
            "redirect_uri": redirect_uri,
            "response_type": "code",
            "state": state,
            "scope": self.config["scope"],
            "duration": "permanent",
        }
        return f"{self.authorize_url}?{urlencode(params)}"


class OAuth2ProviderFactory:
    """Factory class to get the OAuth2 provider."""

    @staticmethod
    def get_provider(provider_name: str) -> OAuth2Provider:
        """Get the OAuth2 provider.

        Args:
            provider_name (str): The name of the provider.

        Returns:
            OAuth2Provider: The OAuth2 provider.

        """
        providers = {
            "twitter": TwitterOAuth2Provider,
            "linkedin": LinkedInOAuth2Provider,
            "github": GitHubOAuth2Provider,
            "reddit": RedditOAuth2Provider,
        }
        provider_class = providers.get(provider_name)
        if not provider_class:
            msg = f"Unsupported provider: {provider_name}"
            raise ValueError(msg)
        return provider_class(provider_name)



import json
import logging

import requests

logger = logging.getLogger(__name__)


class TwitterError(Exception):
    """Exception raised when a post cannot be published to Twitter."""


def post_to_twitter(content: str, access_token: str) -> str:
    """Post a tweet to Twitter.

    Args:
    ----
        content (str): The content of the tweet.
        access_token (str): The access token for the Twitter account.

    Returns:
    -------
        str: The URL of the published tweet.

    """
    logger.debug("Posting to Twitter")

    url = "https://api.twitter.com/2/tweets"
    headers = {
        "Authorization": f"Bearer {access_token}",
        "Content-Type": "application/json",
    }
    payload = {"text": content}

    response = requests.post(url, json=payload, headers=headers, timeout=30)

    if response.status_code == 201:  # noqa: PLR2004
        tweet_data = response.json()
        tweet_id = tweet_data["data"]["id"]
        return f"https://twitter.com/user/status/{tweet_id}"

    msg = f"Failed to post tweet {response.text}"
    raise TwitterError(msg)


class PublishLinkedInError(Exception):
    """Exception raised when a post cannot be published to LinkedIn."""


def publish_to_linkedin(urn: str, access_token: str, content: str) -> str:
    """Publish a post to LinkedIn.

    Args:
    ----
        urn (str): The URN of the LinkedIn account.
        access_token (str): The access token for the LinkedIn account.
        content (str): The content of the post.

    Returns:
    -------
        str: The URL of the published post.

    """
    logger.debug("Publishing to LinkedIn")
    url = "https://api.linkedin.com/v2/ugcPosts"

    headers = {
        "Authorization": f"Bearer {access_token}",
        "X-Restli-Protocol-Version": "2.0.0",
        "Content-Type": "application/json",
    }
    payload = {
        "author": urn,
        "lifecycleState": "PUBLISHED",
        "specificContent": {
            "com.linkedin.ugc.ShareContent": {
                "shareCommentary": {"text": content},
                "shareMediaCategory": "NONE",
            }
        },
        "visibility": {"com.linkedin.ugc.MemberNetworkVisibility": "PUBLIC"},
    }

    r = requests.post(url, headers=headers, data=json.dumps(payload), timeout=30)
    try:
        r.raise_for_status()
    except requests.exceptions.RequestException as e:
        msg = "Could not publish to LinkedIn"
        logger.exception("Response Status Code: %s", r.status_code)
        logger.exception("Response Content: %s", r.content)

        raise PublishLinkedInError(msg) from e

    output = r.json()["id"]
    return f"https://www.linkedin.com/feed/update/{output}/"



from django.contrib import admin
from django.urls import include, path
from django.views.generic import TemplateView

urlpatterns = [
    path("admin/", admin.site.urls),
    path("oauth2_capture/", include("oauth2_capture.urls")),
    path("demo/", include("demo.urls"), name="demo"),
    path("", TemplateView.as_view(template_name="index.html"), name="home"),
]



import logging
import uuid

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http import HttpRequest, HttpResponse
from django.shortcuts import get_object_or_404, redirect, render
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from services.oauth2 import OAuth2ProviderFactory

from demo.services import post_to_twitter, publish_to_linkedin
from oauth2_capture.models import OAuthToken

logger = logging.getLogger(__name__)


@login_required
def index(request: HttpRequest) -> HttpResponse:
    """Render the index page."""
    tokens = OAuthToken.objects.filter(owner=request.user)
    return render(request, "demo.html", {"tokens": tokens})


@require_POST
@csrf_exempt
@login_required
def x_post(request: HttpRequest, slug: str) -> HttpResponse:
    """Handle POST requests."""
    token = get_object_or_404(OAuthToken, slug=slug)
    oa2 = OAuth2ProviderFactory.get_provider("twitter").get_valid_token(token)
    junk = uuid.uuid4().hex
    url = post_to_twitter(f"Hello, world: {junk}", oa2)
    msg = f"Tweet created: {url}"
    logger.info(msg)
    messages.success(request, msg)
    return redirect("/")


@require_POST
@csrf_exempt
@login_required
def linkedin_post(request: HttpRequest, slug: str) -> HttpResponse:
    """Create a linkedin post. There are no refresh tokens available for linkedin. re-auth after 60 days."""
    token = get_object_or_404(OAuthToken, slug=slug)
    oa2 = OAuth2ProviderFactory.get_provider("linkedin").get_valid_token(token)
    url = publish_to_linkedin(
        urn=f"urn:li:person:{token.user_id}",
        access_token=oa2,
        content="Hello, world!",
    )

    messages.success(request, f"Linkedin Post created: {url} ")
    return redirect("/")



from django.urls import path

from oauth2_capture.views import (
    DeleteOAuthTokenView,
    OAuthListView,
    initiate_oauth2,
    oauth2_callback,
)

app_name = "oauth2_capture"

urlpatterns = [
    path("", OAuthListView.as_view(), name="list"),
    path("<slug:slug>/revoke/", DeleteOAuthTokenView.as_view(), name="revoke"),
    path("<str:provider>/connect/", initiate_oauth2, name="initiate_oauth2"),
    path("<str:provider>/callback/", oauth2_callback, name="oauth2_callback"),
]



from django.contrib import admin

from oauth2_capture.models import OAuthToken


@admin.register(OAuthToken)
class OAuthTokenAdmin(admin.ModelAdmin):
    """Admin view for OauthToken model."""

    list_display = ("owner", "provider", "name", "created_at")
    readonly_fields = ("owner", "name")



import os
from pathlib import Path

import environ

env = environ.Env(
    # set casting, default value
    DEBUG=(bool, False)
)

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

# Take environment variables from .env file
environ.Env.read_env(BASE_DIR / "env")


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/5.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.environ["SECRET_KEY"]

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ["*"]


# Application definition

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "oauth2_capture",
    "demo",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "development.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [BASE_DIR / "templates"],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "development.wsgi.application"


# Database
# https://docs.djangoproject.com/en/5.1/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": BASE_DIR / "db.sqlite3",
    }
}


# Password validation
# https://docs.djangoproject.com/en/5.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/5.1/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/5.1/howto/static-files/

STATIC_URL = "static/"

# Default primary key field type
# https://docs.djangoproject.com/en/5.1/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

LOGIN_REDIRECT_URL = "/admin/login/"

# logging config to show debug messages
LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "verbose": {
            "format": "%(name)s : %(levelname)s %(asctime)s %(message)s [%(pathname)s:%(lineno)d]"
        },
    },
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
            "formatter": "verbose",
        },
    },
    "loggers": {
        "": {
            "handlers": ["console"],
            "level": "DEBUG",
            "propagate": False,
        },
    },
}

# settings.py

OAUTH2_CONFIG = {
    "twitter": {
        "client_id": os.environ["TWITTER_CLIENT_ID"],
        "client_secret": os.environ["TWITTER_CLIENT_SECRET"],
        "scope": "tweet.read users.read tweet.write offline.access",
        "code_verifier": "challenge",
    },
    "linkedin": {
        "client_id": os.environ["LINKEDIN_CLIENT_ID"],
        "client_secret": os.environ["LINKEDIN_CLIENT_SECRET"],
        "scope": "profile email openid w_member_social",
    },
    "github": {
        "client_id": os.environ["GITHUB_CLIENT_ID"],
        "client_secret": os.environ["GITHUB_CLIENT_SECRET"],
        "scope": "user repo issues write:discussion",
    },
    "reddit": {
        "client_id": os.environ["REDDIT_CLIENT_ID"],
        "client_secret": os.environ["REDDIT_CLIENT_SECRET"],
        "scope": "identity edit read submit save",
    },
}



# Generated by Django 5.1.1 on 2024-09-23 11:23

import django.db.models.deletion
import shortuuid.django_fields
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="OAuthToken",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "provider",
                    models.CharField(
                        help_text="The name of the OAuth provider (e.g., 'google', 'github', 'facebook').",
                        max_length=50,
                    ),
                ),
                (
                    "slug",
                    shortuuid.django_fields.ShortUUIDField(
                        alphabet=None, length=22, max_length=22, prefix=""
                    ),
                ),
                (
                    "access_token",
                    models.CharField(
                        help_text="The OAuth access token for API access.",
                        max_length=500,
                    ),
                ),
                (
                    "expires_at",
                    models.DateTimeField(
                        blank=True,
                        help_text="The exact expiration time of the access token.",
                        null=True,
                    ),
                ),
                (
                    "refresh_token",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="The refresh token, if provided by the OAuth provider.",
                        max_length=500,
                    ),
                ),
                (
                    "refresh_token_expires_at",
                    models.DateTimeField(
                        blank=True,
                        help_text="The expiration time of the refresh token.",
                        null=True,
                    ),
                ),
                (
                    "token_type",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="Type of the token, typically 'Bearer'.",
                        max_length=50,
                    ),
                ),
                (
                    "scope",
                    models.TextField(
                        blank=True,
                        default="",
                        help_text="Scopes granted by the OAuth provider (e.g., 'email profile').",
                    ),
                ),
                (
                    "user_id",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="The unique ID of the user with the OAuth provider.",
                        max_length=100,
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="The timestamp when the token was issued.",
                    ),
                ),
                ("profile_json", models.JSONField(blank=True, null=True)),
                ("name", models.CharField(blank=True, default="", max_length=100)),
                (
                    "owner",
                    models.ForeignKey(
                        help_text="A reference to the local user this OAuth token is associated with.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="oauth_tokens",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "OAuth Token",
                "verbose_name_plural": "OAuth Tokens",
                "unique_together": {("provider", "user_id")},
            },
        ),
    ]



from django.db import models
from shortuuid.django_fields import ShortUUIDField


class OAuthToken(models.Model):
    """Model to capture the OAuth token for API access."""

    provider = models.CharField(
        max_length=50,
        help_text="The name of the OAuth provider (e.g., 'google', 'github', 'facebook').",
    )
    slug = ShortUUIDField()
    access_token = models.TextField(help_text="The OAuth access token for API access.")
    expires_at = models.DateTimeField(
        null=True,
        blank=True,
        help_text="The exact expiration time of the access token.",
    )
    refresh_token = models.CharField(
        max_length=500,
        blank=True,
        default="",
        help_text="The refresh token, if provided by the OAuth provider.",
    )
    refresh_token_expires_at = models.DateTimeField(
        null=True, blank=True, help_text="The expiration time of the refresh token."
    )
    token_type = models.CharField(
        max_length=50,
        blank=True,
        default="",
        help_text="Type of the token, typically 'Bearer'.",
    )
    scope = models.TextField(
        blank=True,
        default="",
        help_text="Scopes granted by the OAuth provider (e.g., 'email profile').",
    )
    user_id = models.CharField(
        max_length=100,
        blank=True,
        default="",
        help_text="The unique ID of the user with the OAuth provider.",
    )
    owner = models.ForeignKey(
        "auth.User",
        on_delete=models.CASCADE,
        related_name="oauth_tokens",
        help_text="A reference to the local user this OAuth token is associated with.",
    )
    created_at = models.DateTimeField(
        auto_now_add=True, help_text="The timestamp when the token was issued."
    )

    profile_json = models.JSONField(null=True, blank=True)

    name = models.CharField(max_length=100, blank=True, default="")

    class Meta:
        """Metaclass options for the model."""

        unique_together = ("provider", "user_id")
        verbose_name = "OAuth Token"
        verbose_name_plural = "OAuth Tokens"

    def __str__(self) -> str:
        """Return the name of the OAuth token."""
        return f"{self.provider}: {self.name}"

    def is_expired(self) -> bool:
        """Check if the token has expired."""
        from django.utils import timezone

        if self.expires_at:
            return timezone.now() > self.expires_at
        return False

    @property
    def username(self) -> str:
        """Return the username associated with the OAuth token."""
        return (
            self.profile_json.get("username", "")
            or self.profile_json.get("login", "")
            or self.name
        )
